package tetris.model;

import tetris.Tetris;
import tetris.gui.ActionEvent;
import tetris.gui.GUI;
import tetris.model.figures.IFigure;
import tetris.model.figures.JFigure;
import tetris.model.figures.LFigure;
import tetris.model.figures.OFigure;
import tetris.model.figures.SFigure;
import tetris.model.figures.TFigure;
import tetris.model.figures.ZFigure;

import java.util.Random;

/**
 * The class Game implements the Tetris game.
 *
 * @author Stephan Fischli
 * @version 2.1
 */
public class Game {

	/**
	 * The graphical user interface.
	 */
	private final GUI gui;

	/**
	 * The figure of the game.
	 */
	private Figure figure;

	/**
	 * Constructs a game with a graphical user interface.
	 *
	 * @param gui the graphical user interface
	 */
	public Game(GUI gui) {
		this.gui = gui;
	}

	/**
	 * Starts the game by creating a figure and waiting for action events.
	 */
	public void start() {
		createFigure();
		while (true) {
			ActionEvent event = gui.waitEvent();
			handleEvent(event);
		}
	}

	/**
	 * Creates a random figure at the top of the field.
	 */
	private void createFigure() {
		int x = (Tetris.WIDTH - 1) / 2;
		int y = Tetris.HEIGHT - 1;
		int type = (int) (7 * Math.random());
		figure = switch (type) {
			case 0 -> new IFigure(x, y);
			case 1 -> new JFigure(x, y);
			case 2 -> new LFigure(x, y);
			case 3 -> new OFigure(x, y);
			case 4 -> new SFigure(x, y);
			case 5 -> new TFigure(x, y);
			default -> new ZFigure(x, y);
		};
		updateGUI();
	}

	/**
	 * Handles an action event by moving the figure accordingly.
	 *
	 * @param event the event to handle
	 */
	private void handleEvent(ActionEvent event) {
		switch (event) {
			case SHIFT_DOWN -> figure.shift(0, -1);
			case SHIFT_LEFT -> figure.shift(-1, 0);
			case SHIFT_RIGHT -> figure.shift(1, 0);
			case ROTATE_LEFT -> figure.rotate(-1);
			case ROTATE_RIGHT -> figure.rotate(1);
		}
		updateGUI();
	}

	/**
	 * Updates the graphical user interface according to the current state of the game.
	 */
	private void updateGUI() {
		gui.clear();
		gui.drawBlocks(figure.getBlocks());
	}
}
